import unittest
from netils import build_byte_string
from chewie.message_parser import MessageParser, MessagePacker, IdentityMessage, Md5ChallengeMessage
from chewie.message_parser import EapolStartMessage, EapolLogoffMessage, SuccessMessage, FailureMessage
from chewie.mac_address import MacAddress
from chewie.eap import Eap

class EapTestCase(unittest.TestCase):
    def test_identity_request_message_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e010000050101000501000000")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:19:06:ea:b8:8c"), message.src_mac)
        self.assertEqual(1, message.message_id)
        self.assertEqual(Eap.REQUEST, message.code)
        self.assertEqual("", message.identity)

    def test_identity_response_message_parses(self):
        packed_message = build_byte_string("0180c2000003001422e9545e888e0100001102000011014a6f686e2e4d63477569726b")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:14:22:e9:54:5e"), message.src_mac)
        self.assertEqual(0, message.message_id)
        self.assertEqual(Eap.RESPONSE, message.code)
        self.assertEqual("John.McGuirk", message.identity)

    def test_md5_challenge_request_message_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e01000016010100160410824788d693e2adac6ce15641418228cf0000")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:19:06:ea:b8:8c"), message.src_mac)
        self.assertEqual(1, message.message_id)
        self.assertEqual(Eap.REQUEST, message.code)
        self.assertEqual(build_byte_string("824788d693e2adac6ce15641418228cf"), message.challenge)
        self.assertEqual(b"", message.extra_data)

    def test_md5_challenge_response_message_parses(self):
        packed_message = build_byte_string("0180c2000003001422e9545e888e010000220201002204103a535f0ee8c6b34fe714aa7dad9a0e154a6f686e2e4d63477569726b")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:14:22:e9:54:5e"), message.src_mac)
        self.assertEqual(1, message.message_id)
        self.assertEqual(Eap.RESPONSE, message.code)
        self.assertEqual(build_byte_string("3a535f0ee8c6b34fe714aa7dad9a0e15"), message.challenge)
        self.assertEqual(b"John.McGuirk", message.extra_data)

    def test_identity_request_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e010000050101000501")
        message = IdentityMessage(src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"), message_id=1, code=Eap.REQUEST, identity="")
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_identity_response_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001422e9545e888e0100001102000011014a6f686e2e4d63477569726b")
        message = IdentityMessage(src_mac=MacAddress.from_string("00:14:22:e9:54:5e"), message_id=0, code=Eap.RESPONSE, identity="John.McGuirk")
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_md5_challenge_request_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e01000016010100160410824788d693e2adac6ce15641418228cf")
        message = Md5ChallengeMessage(src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"), message_id=1, code=Eap.REQUEST, challenge=build_byte_string("824788d693e2adac6ce15641418228cf"), extra_data=b"")
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_md5_challenge_response_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001422e9545e888e010000220201002204103a535f0ee8c6b34fe714aa7dad9a0e154a6f686e2e4d63477569726b")
        message = Md5ChallengeMessage(src_mac=MacAddress.from_string("00:14:22:e9:54:5e"), message_id=1, code=Eap.RESPONSE, challenge=build_byte_string("3a535f0ee8c6b34fe714aa7dad9a0e15"), extra_data=b"John.McGuirk")
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_eapol_start_message_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e01010000")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:19:06:ea:b8:8c"), message.src_mac)
        self.assertTrue(isinstance(message, EapolStartMessage))

    def test_eapol_start_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e01010000")
        message = EapolStartMessage(src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"))
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_eapol_logoff_message_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e01020000")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:19:06:ea:b8:8c"), message.src_mac)
        self.assertTrue(isinstance(message, EapolLogoffMessage))

    def test_eapol_logoff_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e01020000")
        message = EapolLogoffMessage(src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"))
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_success_message_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e0100000403ff0004")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:19:06:ea:b8:8c"), message.src_mac)
        self.assertEqual(255, message.message_id)
        self.assertTrue(isinstance(message, SuccessMessage))

    def test_success_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e0100000403ff0004")
        message = SuccessMessage(src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"), message_id=255)
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)

    def test_failure_message_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e0100000404ff0004")
        message = MessageParser.parse(packed_message)
        self.assertEqual(MacAddress.from_string("00:19:06:ea:b8:8c"), message.src_mac)
        self.assertEqual(255, message.message_id)
        self.assertTrue(isinstance(message, FailureMessage))

    def test_failure_message_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e0100000404ff0004")
        message = FailureMessage(src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"), message_id=255)
        packed_message = MessagePacker.pack(message)
        self.assertEqual(expected_packed_message, packed_message)



from .ethernet_packet import EthernetPacket
from .auth_8021x import Auth8021x
from .eap import Eap, EapIdentity, EapMd5Challenge, EapSuccess, EapFailure
from .mac_address import MacAddress

class SuccessMessage(object):
    def __init__(self, src_mac, message_id):
        self.src_mac = src_mac
        self.message_id = message_id

    @classmethod
    def build(cls, src_mac, eap):
        return cls(src_mac, eap.packet_id)

class FailureMessage(object):
    def __init__(self, src_mac, message_id):
        self.src_mac = src_mac
        self.message_id = message_id

    @classmethod
    def build(cls, src_mac, eap):
        return cls(src_mac, eap.packet_id)

class IdentityMessage(object):
    def __init__(self, src_mac, message_id, code, identity):
        self.src_mac = src_mac
        self.message_id = message_id
        self.code = code
        self.identity = identity

    @classmethod
    def build(cls, src_mac, eap):
        return cls(src_mac, eap.packet_id, eap.code, eap.identity)

class Md5ChallengeMessage(object):
    def __init__(self, src_mac, message_id, code, challenge, extra_data):
        self.src_mac = src_mac
        self.message_id = message_id
        self.code = code
        self.challenge = challenge
        self.extra_data = extra_data

    @classmethod
    def build(cls, src_mac, eap):
        return cls(src_mac, eap.packet_id, eap.code, eap.challenge, eap.extra_data)

class EapolStartMessage(object):
    def __init__(self, src_mac):
        self.src_mac = src_mac

    @classmethod
    def build(cls, src_mac):
        return cls(src_mac)

class EapolLogoffMessage(object):
    def __init__(self, src_mac):
        self.src_mac = src_mac

    @classmethod
    def build(cls, src_mac):
        return cls(src_mac)

EAP_MESSAGES = {
    Eap.IDENTITY: IdentityMessage,
    Eap.MD5_CHALLENGE: Md5ChallengeMessage,
}

AUTH_8021X_MESSAGES = {
    0: "eap",
    1: "eapol start",
}

class MessageParser:
    @staticmethod
    def parse(packed_message):
        ethernet_packet = EthernetPacket.parse(packed_message)
        if ethernet_packet.ethertype != 0x888e:
            raise ValueError("Ethernet packet with bad ethertype received: %s" % ethernet_packet)
        auth_8021x = Auth8021x.parse(ethernet_packet.data)
        if auth_8021x.packet_type == 0:
            eap = Eap.parse(auth_8021x.data)
            if isinstance(eap, EapIdentity) or isinstance(eap, EapMd5Challenge):
                return EAP_MESSAGES[eap.PACKET_TYPE].build(ethernet_packet.src_mac, eap)
            elif isinstance(eap, EapSuccess):
                return SuccessMessage.build(ethernet_packet.src_mac, eap)
            elif isinstance(eap, EapFailure):
                return FailureMessage.build(ethernet_packet.src_mac, eap)
            else:
                raise ValueError("Got bad Eap packet: %s" % eap)
        elif auth_8021x.packet_type == 1:
            return EapolStartMessage.build(ethernet_packet.src_mac)
        elif auth_8021x.packet_type == 2:
            return EapolLogoffMessage.build(ethernet_packet.src_mac)
        raise ValueError("802.1x has bad type, expected 0: %s" % auth_8021x)

class MessagePacker:
    @staticmethod
    def pack(message):
        if isinstance(message, IdentityMessage):
            eap = EapIdentity(message.code, message.message_id, message.identity)
            auth_8021x = Auth8021x(version=1, packet_type=0, data=eap.pack())
        elif isinstance(message, Md5ChallengeMessage):
            eap = EapMd5Challenge(message.code, message.message_id, message.challenge, message.extra_data)
            auth_8021x = Auth8021x(version=1, packet_type=0, data=eap.pack())
        elif isinstance(message, SuccessMessage):
            eap = EapSuccess(message.message_id)
            auth_8021x = Auth8021x(version=1, packet_type=0, data=eap.pack())
        elif isinstance(message, FailureMessage):
            eap = EapFailure(message.message_id)
            auth_8021x = Auth8021x(version=1, packet_type=0, data=eap.pack())
        elif isinstance(message, EapolStartMessage):
            auth_8021x = Auth8021x(version=1, packet_type=1, data=b"")
        elif isinstance(message, EapolLogoffMessage):
            auth_8021x = Auth8021x(version=1, packet_type=2, data=b"")
        else:
            raise ValueError("Cannot pack message: %s" % message)
        ethernet_packet = EthernetPacket(dst_mac=MacAddress.from_string("01:80:c2:00:00:03"), src_mac=message.src_mac, ethertype=0x888e, data=auth_8021x.pack())
        return ethernet_packet.pack()




import unittest
import struct

from netils import build_byte_string
from hashlib import md5

from chewie.state_machine import StateMachine
from chewie.mac_address import MacAddress
from chewie.message_parser import IdentityMessage, Md5ChallengeMessage, EapolStartMessage
from chewie.event import EventMessageReceived
from chewie.eap import Eap

def txn_id_injector():
    return 123

def challenge_injector(_):
    return build_byte_string("01234567890abcdef01234567890abcdef")

def build_state_machine(src_mac):
    state_machine = StateMachine(src_mac)
    state_machine.txn_id_method = txn_id_injector
    state_machine.challenge_method = challenge_injector

    return state_machine

class StateMachineIdleTestCase(unittest.TestCase):
    def setUp(self):
        src_mac = MacAddress.from_string("00:aa:bb:cc:dd:ee")
        self.state_machine = build_state_machine(src_mac)
        self.assertEqual(self.state_machine.state, "idle")

    def test_packet_received_moves_to_authing_state(self):
        message = EapolStartMessage(MacAddress.from_string("00:12:34:56:78:90"))
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "identity request sent")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)

class StateMachineIdentitySentTestCase(unittest.TestCase):
    def setUp(self):
        src_mac = MacAddress.from_string("00:aa:bb:cc:dd:ee")
        self.state_machine = build_state_machine(src_mac)
        self.assertEqual(self.state_machine.state, "idle")
        message = EapolStartMessage(MacAddress.from_string("00:12:34:56:78:90"))
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "identity request sent")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)
        self.state_machine.output_messages.get()

    def test_identity_request_received_moves_to_challenge_sent(self):
        message = IdentityMessage(MacAddress.from_string("00:12:34:56:78:90"), 1, Eap.RESPONSE, "betelgeuse")
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "challenge sent")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)

class StateMachineChallengeSentTestCase(unittest.TestCase):
    def setUp(self):
        src_mac = MacAddress.from_string("00:aa:bb:cc:dd:ee")
        self.state_machine = build_state_machine(src_mac)
        self.assertEqual(self.state_machine.state, "idle")
        message = EapolStartMessage(MacAddress.from_string("00:12:34:56:78:90"))
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "identity request sent")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)
        self.state_machine.output_messages.get()
        message = IdentityMessage(MacAddress.from_string("00:12:34:56:78:90"), 1, Eap.RESPONSE, "betelgeuse")
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "challenge sent")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)
        self.state_machine.output_messages.get()

    def test_correct_challenge_received_moves_to_authenticated(self):
        txn_id = 123
        challenge = build_byte_string("01234567890abcdef01234567890abcdef")
        password = "microphone"
        id_string = struct.pack("B", txn_id)
        challenge_response = md5(id_string + password.encode() + challenge).digest()
        message = Md5ChallengeMessage(
            MacAddress.from_string("00:12:34:56:78:90"), self.state_machine.txn_id, Eap.RESPONSE, challenge_response, b"who cares")
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "authenticated")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)

    def test_incorrect_challenge_received_moves_to_idle(self):
        txn_id = 123
        challenge = build_byte_string("01234567890abcdef01234567890abcdef")
        password = "notmicrophone"
        id_string = struct.pack("B", txn_id)
        challenge_response = md5(id_string + password.encode() + challenge).digest()
        message = Md5ChallengeMessage(
            MacAddress.from_string("00:12:34:56:78:90"), self.state_machine.txn_id, Eap.RESPONSE, challenge_response, b"who cares")
        self.state_machine.event(EventMessageReceived(message))
        self.assertEqual(self.state_machine.state, "idle")
        self.assertEqual(self.state_machine.output_messages.qsize(), 1)



import struct
from netils import build_byte_string

class MacAddress:
    def __init__(self, address):
        self.address = address

    @classmethod
    def from_string(cls, address_string):
        address_bytes = "".join(address_string.split(":"))
        address = build_byte_string(address_bytes)

        return cls(address)

    def __str__(self):
        address_string = ":".join("%02x" % x for x in self.address)
        return address_string

    def __eq__(self, other):
        return self.address == other.address

    def __hash__(self):
        return hash(self.address)

    def __repr__(self):
        return "%s.from_string(\"%s\")" % (self.__class__.__name__, self.__str__())



import os
import struct

from hashlib import md5
from eventlet.queue import Queue
from .event import EventMessageReceived
from .message_parser import IdentityMessage, Md5ChallengeMessage, EapolStartMessage
from .message_parser import SuccessMessage, FailureMessage
from .eap import Eap

def generate_txn_id():
    return ord(os.urandom(1))

def generate_random_bytes(num_bytes):
    return os.urandom(num_bytes)

class StateMachine:
    def __init__(self, src_mac):
        self.src_mac = src_mac

        self.txn_id = None
        self.challenge = None
        self.expected_response = None
        # TODO - some way to query for this based on identity
        self.password = "microphone"
        self.state = "idle"
        self.output_messages = Queue()

        self.txn_id_method = generate_txn_id
        self.challenge_method = generate_random_bytes

    def event(self, event):
        if isinstance(event, EventMessageReceived):
            self.handle_message_received(event.message)

    def handle_message_received(self, message):
        if self.state == "idle":
            self.handle_idle_message(message)
        elif self.state == "identity request sent":
            self.handle_identity_sent_message(message)
        elif self.state == "challenge sent":
            self.handle_challenge_sent_message(message)

    def handle_idle_message(self, message):
        if isinstance(message, EapolStartMessage):
            self.txn_id = self.txn_id_method()
            identity_request = IdentityMessage(self.src_mac, self.txn_id, Eap.REQUEST, "")
            self.output_messages.put(identity_request)
            self.state = "identity request sent"

    def handle_identity_sent_message(self, message):
        if isinstance(message, IdentityMessage):
            self.challenge = self.challenge_method(16)
            self.calculate_expected_response()
            challenge_request = Md5ChallengeMessage(self.src_mac, self.txn_id, Eap.REQUEST, self.challenge, b"")
            self.output_messages.put(challenge_request)
            self.state = "challenge sent"

    def handle_challenge_sent_message(self, message):
        if isinstance(message, Md5ChallengeMessage):
            if message.challenge == self.expected_response:
                message = SuccessMessage(self.src_mac, self.txn_id)
                self.output_messages.put(message)
                self.state = "authenticated"
            else:
                message = FailureMessage(self.src_mac, self.txn_id)
                self.output_messages.put(message)
                self.state = "idle"

    def calculate_expected_response(self):
        txn_id_string = struct.pack("B", self.txn_id)
        self.expected_response = md5(txn_id_string + self.password.encode() + self.challenge).digest()



class Event(object):
    TIMER_EXPIRED = 1
    MESSAGE_RECEIVED = 2
    SHUTDOWN = 3

class EventTimerExpired(Event):
    def __init__(self):
        # will work but please do this properly
        self.type = self.TIMER_EXPIRED

class EventMessageReceived(Event):
    def __init__(self, message):
        # will work but please do this properly
        self.type = self.MESSAGE_RECEIVED
        self.message = message

class EventShutdown(Event):
    def __init__(self):
        # will work but please do this properly
        self.type = self.SHUTDOWN



from eventlet import sleep, GreenPool
from eventlet.queue import Queue
import eventlet.greenthread as greenthread
import socket
import struct
from hashlib import md5
from fcntl import ioctl
from netils import build_byte_string

from .ethernet_packet import EthernetPacket
from .auth_8021x import Auth8021x
from .eap import Eap, EapIdentity, EapMd5Challenge
from .message_parser import MessageParser, MessagePacker, IdentityMessage, Md5ChallengeMessage
from .mac_address import MacAddress
from .state_machine import StateMachine
from .event import EventMessageReceived

def unpack_byte_string(byte_string):
    return "".join("%02x" % x for x in byte_string)

class Chewie(object):
    SIOCGIFHWADDR = 0x8927
    SIOCGIFINDEX = 0x8933
    PACKET_MR_MULTICAST = 0
    SOL_PACKET = 263
    PACKET_ADD_MEMBERSHIP = 1
    EAP_ADDRESS = build_byte_string("0180c2000003")

    def __init__(self, interface_name, credentials):
        self.interface_name = interface_name
        self.credentials = credentials

    def run(self):
        self.open_socket()
        self.get_interface_info()
        self.build_state_machine()
        self.join_multicast_group()
        #self.run_demo()
        self.start_threads_and_wait()

    def start_threads_and_wait(self):
        print("Starting threads")
        self.pool = GreenPool()
        self.eventlets = []

        self.eventlets.append(self.pool.spawn(self.send_messages))
        self.eventlets.append(self.pool.spawn(self.receive_messages))

        self.pool.waitall()

    def send_messages(self):
        while True:
            sleep(0)
            message = self.state_machine.output_messages.get()
            print("Sending message: %s" % message)
            self.socket.send(MessagePacker.pack(message))

    def receive_messages(self):
        while True:
            sleep(0)
            packed_message = self.socket.recv(4096)
            message = MessageParser.parse(packed_message)
            print("Received message: %s" % message)
            event = EventMessageReceived(message)
            self.state_machine.event(event)

    def run_demo(self):
        self.seed = md5("banana".encode()).digest()
        print("Sending packet")
        self.message_id = 123
        packet = MessagePacker.pack(IdentityMessage(self.interface_address, self.message_id, Eap.REQUEST, ""))
        self.socket.send(packet)
        response = self.socket.recv(4096)
        self.handle_eap_packet(response)
        packet = MessagePacker.pack(Md5ChallengeMessage(self.interface_address, self.message_id, Eap.REQUEST, self.seed, b""))
        self.socket.send(packet)
        response = self.socket.recv(4096)
        self.handle_eap_packet(response)
        # send success to keep it happy
        success = build_byte_string("888e0100000403010004")
        packet = self.EAP_ADDRESS + self.interface_address.address + success
        self.socket.send(packet)

        self.socket.close()

    def handle_eap_packet(self, packed_message):
        print("packed message: %s" % packed_message)
        message = MessageParser.parse(packed_message)
        if isinstance(message, IdentityMessage):
            print("Eap packet type identity")
            print("Identity: %s" % message.identity)
        elif isinstance(message, Md5ChallengeMessage):
            print("Eap packet type md5-challenge")
            print("Response: %s" % unpack_byte_string(message.challenge))
            password="microphone".encode()
            challenge_id_string = struct.pack("B", self.message_id)
            expected_response = md5(challenge_id_string + password + self.seed).digest()
            print("Expected response: %s" % unpack_byte_string(expected_response))
        else:
            print("Unknown message %s" % message)

    def open_socket(self):
        self.socket = socket.socket(socket.PF_PACKET, socket.SOCK_RAW, socket.htons(0x888e))
        self.socket.bind((self.interface_name, 0))

    def build_state_machine(self):
        self.state_machine = StateMachine(self.interface_address)

    def get_interface_info(self):
        self.get_interface_address()
        self.get_interface_index()

    def get_interface_address(self):
        # http://man7.org/linux/man-pages/man7/netdevice.7.html
        ifreq = struct.pack('16sH6s', self.interface_name.encode("utf-8"), 0, b"")
        response = ioctl(self.socket, self.SIOCGIFHWADDR, ifreq)
        _interface_name, _address_family, interface_address = struct.unpack('16sH6s', response)
        self.interface_address = MacAddress(interface_address)

    def get_interface_index(self):
        # http://man7.org/linux/man-pages/man7/netdevice.7.html
        ifreq = struct.pack('16sI', self.interface_name.encode("utf-8"), 0)
        response = ioctl(self.socket, self.SIOCGIFINDEX, ifreq)
        _ifname, self.interface_index = struct.unpack('16sI', response)

    def join_multicast_group(self):
        mreq = struct.pack("IHH8s", self.interface_index, self.PACKET_MR_MULTICAST, len(self.EAP_ADDRESS), self.EAP_ADDRESS)
        self.socket.setsockopt(self.SOL_PACKET, self.PACKET_ADD_MEMBERSHIP, mreq)



class Timer:
    def __init__(self, count):
        self.count = count
        self.tick = None

    def running(self):
        return bool(self.tick)

    def expired(self, tick):
        return self.running() and tick > self.tick + self.count

    def reset(self, tick):
        self.tick = tick

    def stop(self):
        self.tick = None



import unittest
from netils import build_byte_string
from chewie.ethernet_packet import EthernetPacket
from chewie.mac_address import MacAddress

class EthernetPacketTestCase(unittest.TestCase):
    def test_ethernet_packet_parses(self):
        packed_message = build_byte_string("0180c2000003001906eab88c888e0100000501010005010000")
        message = EthernetPacket.parse(packed_message)
        self.assertEqual(message.src_mac, MacAddress.from_string("00:19:06:ea:b8:8c"))
        self.assertEqual(message.dst_mac, MacAddress.from_string("01:80:c2:00:00:03"))
        self.assertEqual(message.ethertype, 0x888e)
        self.assertEqual(message.data, build_byte_string("0100000501010005010000"))

    def test_ethernet_packet_packs(self):
        expected_packed_message = build_byte_string("0180c2000003001906eab88c888e0100000501010005010000")
        message = EthernetPacket(dst_mac=MacAddress.from_string("01:80:c2:00:00:03"), src_mac=MacAddress.from_string("00:19:06:ea:b8:8c"), ethertype=0x888e, data=build_byte_string("0100000501010005010000"))
        packed_message = message.pack()
        self.assertEqual(expected_packed_message, packed_message)



import unittest

from chewie.timer import Timer

class TimerTestCase(unittest.TestCase):
    def test_timer_is_off_by_default(self):
        timer = Timer(60)
        self.assertFalse(timer.running())

    def test_timer_expires(self):
        base_tick = 10000
        timer_count = 60
        timer = Timer(timer_count)
        self.assertFalse(timer.expired(base_tick))
        self.assertFalse(timer.expired(base_tick+timer_count-1))
        self.assertFalse(timer.expired(base_tick+timer_count+1))
        timer.reset(base_tick)
        self.assertTrue(timer.running())
        self.assertFalse(timer.expired(base_tick))
        self.assertFalse(timer.expired(base_tick+timer_count-1))
        self.assertTrue(timer.expired(base_tick+timer_count+1))
        timer.stop()
        self.assertFalse(timer.expired(base_tick))
        self.assertFalse(timer.expired(base_tick+timer_count-1))
        self.assertFalse(timer.expired(base_tick+timer_count+1))



from setuptools import setup

long_description = """
    Chewie is an EAPOL/802.1x implementation in Python.

    Currently barely works, useful if you have one user and their password is "microphone"

    More information at https://github.com/samrussell/chewie
"""

setup(
    name='chewie',
    description='A bare-bones EAPOL/802.1x implementation',
    long_description=long_description,
    version='0.0.1',
    url='https://github.com/samrussell/chewie',
    author='Sam Russell',
    author_email='sam.h.russell@gmail.com',
    license='Apache2',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3'
    ],
    keywords='802.1x dot1x eap eapol radius authentication aaa',
    packages=['chewie'],
    python_requires='>=3',
    install_requires=[
        'eventlet',
        'netils==0.0.1'
    ]
)



import struct

EAP_HEADER_LENGTH = 1 + 1 + 2
EAP_TYPE_LENGTH = 1

PARSERS = {}

class Eap(object):
    REQUEST = 1
    RESPONSE = 2
    SUCCESS = 3
    FAILURE = 4

    IDENTITY = 1
    MD5_CHALLENGE = 4

    @staticmethod
    def parse(packed_message):
        code, packet_id, length = struct.unpack("!BBH", packed_message[:EAP_HEADER_LENGTH])
        if code == Eap.REQUEST or code == Eap.RESPONSE:
            packet_type, = struct.unpack("!B", packed_message[EAP_HEADER_LENGTH:EAP_HEADER_LENGTH+EAP_TYPE_LENGTH])
            data = packed_message[EAP_HEADER_LENGTH+EAP_TYPE_LENGTH:length]
            return PARSERS[packet_type](code, packet_id, data)
        elif code == Eap.SUCCESS:
            return EapSuccess(packet_id)
        elif code == Eap.FAILURE:
            return EapFailure(packet_id)
        raise ValueError("Got Eap packet with bad code: %s" % packed_message)

    def pack(self, packed_body):
        header = struct.pack("!BBHB", self.code, self.packet_id, EAP_HEADER_LENGTH + EAP_TYPE_LENGTH + len(packed_body), self.PACKET_TYPE)
        return header + packed_body

def register_parser(cls):
    PARSERS[cls.PACKET_TYPE] = cls.parse
    return cls

@register_parser
class EapIdentity(Eap):
    PACKET_TYPE = 1

    def __init__(self, code, packet_id, identity):
        self.code = code
        self.packet_id = packet_id
        self.identity = identity

    @classmethod
    def parse(cls, code, packet_id, packed_message):
        return cls(code, packet_id, packed_message.decode())

    def pack(self):
        packed_identity = self.identity.encode()
        return super(EapIdentity, self).pack(packed_identity)

    def __repr__(self):
        return "%s(identity=%s)" % \
            (self.__class__.__name__, self.identity)

@register_parser
class EapMd5Challenge(Eap):
    PACKET_TYPE = 4

    def __init__(self, code, packet_id, challenge, extra_data):
        self.code = code
        self.packet_id = packet_id
        self.challenge = challenge
        self.extra_data = extra_data

    @classmethod
    def parse(cls, code, packet_id, packed_message):
        value_length, = struct.unpack("!B", packed_message[:1])
        challenge = packed_message[1:1+value_length]
        extra_data = packed_message[1+value_length:]
        return cls(code, packet_id, challenge, extra_data)

    def pack(self):
        value_length = struct.pack("!B", len(self.challenge))
        packed_md5_challenge = value_length + self.challenge + self.extra_data
        return super(EapMd5Challenge, self).pack(packed_md5_challenge)

    def __repr__(self):
        return "%s(challenge=%s, extra_data=%s)" % \
            (self.__class__.__name__, self.challenge, self.extra_data)

class EapSuccess(Eap):
    def __init__(self, packet_id):
        self.packet_id = packet_id

    @classmethod
    def parse(cls, packet_id):
        return cls(code, packet_id)

    def pack(self):
        return struct.pack("!BBH", Eap.SUCCESS, self.packet_id, EAP_HEADER_LENGTH)

    def __repr__(self):
        return "%s(packet_id=%s)" % \
            (self.__class__.__name__, self.packet_id)

class EapFailure(Eap):
    def __init__(self, packet_id):
        self.packet_id = packet_id

    @classmethod
    def parse(cls, packet_id):
        return cls(code, packet_id)

    def pack(self):
        return struct.pack("!BBH", Eap.FAILURE, self.packet_id, EAP_HEADER_LENGTH)

    def __repr__(self):
        return "%s(packet_id=%s)" % \
            (self.__class__.__name__, self.packet_id)



import struct
from .mac_address import MacAddress

ETHERNET_HEADER_LENGTH = 6 + 6 + 2

class EthernetPacket(object):
    def __init__(self, dst_mac, src_mac, ethertype, data):
        self.dst_mac = dst_mac
        self.src_mac = src_mac
        self.ethertype = ethertype
        self.data = data

    @classmethod
    def parse(cls, packed_message):
        dst_mac, src_mac, ethertype = struct.unpack("!6s6sH", packed_message[:ETHERNET_HEADER_LENGTH])
        data = packed_message[ETHERNET_HEADER_LENGTH:]
        return cls(MacAddress(dst_mac), MacAddress(src_mac), ethertype, data)

    def pack(self):
        header = struct.pack("!6s6sH", self.dst_mac.address, self.src_mac.address, self.ethertype)
        return header + self.data

    def __repr__(self):
        return "%s(dst_mac=%s, src_mac=%s, ethertype=0x%04X)" % \
            (self.__class__.__name__, self.dst_mac, self.src_mac, self.ethertype)


import struct

def hwaddr_to_string(hwaddr):
    return ":".join(["%02x" % x for x in hwaddr])

AUTH_8021X_HEADER_LENGTH = 1 + 1 + 2

class Auth8021x(object):
    def __init__(self, version, packet_type, data):
        self.version = version
        self.packet_type = packet_type
        self.data = data

    @classmethod
    def parse(cls, packed_message):
        version, packet_type, length = struct.unpack("!BBH", packed_message[:AUTH_8021X_HEADER_LENGTH])
        data = packed_message[AUTH_8021X_HEADER_LENGTH:AUTH_8021X_HEADER_LENGTH+length]
        return cls(version, packet_type, data)

    def pack(self):
        header = struct.pack("!BBH", self.version, self.packet_type, len(self.data))
        return header + self.data

    def __repr__(self):
        return "%s(ethernet_header=%s, version=%s, packet_type=%s, data=%s)" % \
            (self.__class__.__name__, self.version, self.packet_type, self.data)

    def __str__(self):
        return "%s<packet_type=%d, data=%s>" % \
            (self.__class__.__name__, self.packet_type, self.data)
